// DDS defines for AVR PWM-based DDS
// $Id: dds.h,v 1.2 2007/12/12 22:18:47 cm Exp $

#define DDS_F_S (F_CPU/512)	// DDS base sample rate, 15625 Hz
#define DDS_NUM_CHANNELS 2

// phase accumulator and tuning for a number of channels
uint32_t dds_pa[DDS_NUM_CHANNELS];
volatile uint32_t dds_tune[DDS_NUM_CHANNELS];

// calculate tuning for a certain frequency
// 1<<26: we have a 26 bit phase accumulator
#define DDS_TUNE(freq) (uint32_t)((freq)/DDS_F_S*(1UL<<26))

#include "sinetable.h"

#if SINE_LUT_SIZE != 256
#error SINE_LUT_SIZE must be 256 (sinetable.h)
#endif


static uint8_t dds(uint8_t chan) {
  uint8_t idx, magic;

  idx = (dds_pa[chan] & 0xff0000UL) >> 16; // into 1/4 sine table
  magic = (dds_pa[chan] & 0x03000000UL) >> 24; // two magic bits

  dds_pa[chan] += dds_tune[chan]; // rotate...

  if(magic & 1) idx = 255 - idx; // mirrored along y axis

  if(magic & 2)			// mirrored along x axis
    return(128 - pgm_read_byte(&sine_lut[idx]));
  else
    return(128 + pgm_read_byte(&sine_lut[idx]));
}

/*
  $Log: dds.h,v $
  Revision 1.2  2007/12/12 22:18:47  cm
  prettify

*/
